package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.math.BigDecimal;
import java.util.Map;

/**
 * Sample code for the CreateShipment Canada Post service.
 * <p>
 * The CreateShipment service is used to initiate generation of a shipping label
 * by providing shipment details. Use of this service indicates an intention to
 * pay for shipment of an item.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 *
 * For local testing only.
 */
public class CreateShipmentClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());


            //build request
            ObjectFactory of = new ObjectFactory();

            AddressDetailsType senderAddressDetailsType = of.createAddressDetailsType();
            senderAddressDetailsType.setAddressLine1("502 MAIN ST N");
            senderAddressDetailsType.setCity("MONTREAL");
            senderAddressDetailsType.setProvState("QC");
            senderAddressDetailsType.setCountryCode("CA");
            senderAddressDetailsType.setPostalZipCode("H2B1A0");

            SenderType senderType = of.createSenderType();
            senderType.setName("Bulma");
            senderType.setCompany("Capsule Corp.");
            senderType.setContactPhone("1 (514) 820 5879");
            senderType.setAddressDetails(senderAddressDetailsType);

            DestinationAddressDetailsType destAddressDetailsType = of.createDestinationAddressDetailsType();
            destAddressDetailsType.setAddressLine1("123 Postal Drive");
            destAddressDetailsType.setCity("Ottawa");
            destAddressDetailsType.setProvState("ON");
            destAddressDetailsType.setCountryCode("CA");
            destAddressDetailsType.setPostalZipCode("K1P5Z9");

            DestinationType destinationType = of.createDestinationType();
            destinationType.setName("John Doe");
            destinationType.setCompany("ACME Corp.");
            destinationType.setAddressDetails(destAddressDetailsType);

            OptionType optionType = of.createOptionType();
            optionType.setOptionCode("DC");

            DeliverySpecType.Options options = of.createDeliverySpecTypeOptions();
            options.getOption().add(optionType);

            ParcelCharacteristicsType.Dimensions dimensions = of.createParcelCharacteristicsTypeDimensions();
            dimensions.setLength(new BigDecimal(6));
            dimensions.setWidth(new BigDecimal(12));
            dimensions.setHeight(new BigDecimal(9));

            ParcelCharacteristicsType parcelCharacteristicsType = of.createParcelCharacteristicsType();
            parcelCharacteristicsType.setWeight(new BigDecimal(15));
            parcelCharacteristicsType.setUnpackaged(false);
            parcelCharacteristicsType.setMailingTube(false);
            parcelCharacteristicsType.setDimensions(dimensions);

            PrintPreferencesType printPreferencesType = of.createPrintPreferencesType();
            printPreferencesType.setOutputFormat("4x6");
            printPreferencesType.setEncoding("ZPL");

            PreferencesType preferencesType = of.createPreferencesType();
            preferencesType.setShowPackingInstructions(true);
            preferencesType.setShowPostageRate(false);
            preferencesType.setShowInsuredValue(true);

            SettlementInfoType settlementInfoType = of.createSettlementInfoType();
            settlementInfoType.setContractId("0040662505");
            settlementInfoType.setIntendedMethodOfPayment("account");

            ReferencesType referencesType = of.createReferencesType();
            referencesType.setCostCentre("costCentre");
            referencesType.setCustomerRef1("customerReference1");
            referencesType.setCustomerRef2("customerReference2");

            DeliverySpecType deliverySpecType = of.createDeliverySpecType();
            deliverySpecType.setServiceCode("DOM.EP");
            deliverySpecType.setSender(senderType);
            deliverySpecType.setDestination(destinationType);
            deliverySpecType.setOptions(options);
            deliverySpecType.setParcelCharacteristics(parcelCharacteristicsType);
            deliverySpecType.setPrintPreferences(printPreferencesType);
            deliverySpecType.setPreferences(preferencesType);
            deliverySpecType.setSettlementInfo(settlementInfoType);
            deliverySpecType.setReferences(referencesType);

            ShipmentType shipmentType = of.createShipmentType();

            //shipmentType.setGroupIdOrTransmitShipment(of.createTransmitShipment(true));
            shipmentType.setGroupIdOrTransmitShipment(of.createGroupId("groupy"));
            shipmentType.setCpcPickupIndicator(true);
            shipmentType.setRequestedShippingPoint("K1K1K1");
            shipmentType.setExpectedMailingDate(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-06-20"));
            shipmentType.setDeliverySpec(deliverySpecType);

            CreateShipmentRequest request = of.createCreateShipmentRequest();
            request.setMailedBy(mailedBy);
            request.setLocale("FR");
            request.setShipment(shipmentType);


            //execute request
            CreateShipmentResponse response = port.createShipment(request);


            //retrieve values from response
            if(response.getMessages() == null){
                ShipmentInfoDetailType shipmentInfoType = response.getShipmentInfo();

                System.out.println("Shipment Id: " + shipmentInfoType.getShipmentId());
                System.out.println("Tracking Pin: " + shipmentInfoType.getTrackingPin());
                for(ArtifactType a : shipmentInfoType.getArtifacts().getArtifact()){
                    System.out.println("Artifact Type: " + a.getType());
                    System.out.println("Artifact Id: " + a.getArtifactId());
                    System.out.println("Page Index: " + a.getPageIndex());
                    System.out.println();
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
