package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;

import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetShipments Canada Post service.
 * <p>
 * The GetShipments service is used to retrieve the shipment IDs for a collection
 * of shipments not yet transmitted (members of a group), or to retrieve the
 * shipment IDs for a collection of shipments that have been transmitted (members
 * of a manifest).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetShipmentsClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //build request
            ObjectFactory of = new ObjectFactory();

            GetShipmentsRequest.ShipmentSearchKey shipmentSearchKey = of.createGetShipmentsRequestShipmentSearchKey();

            //Get group shipments
            RegularShippingParameters regularShippingParameters = of.createRegularShippingParameters();
            regularShippingParameters.setGroupId("groupy");
            shipmentSearchKey.setRegularShippingParameters(regularShippingParameters);

            //Get manifest shipments
            /*
            RegularShippingParameters regularShippingParameters = of.createRegularShippingParameters();
            regularShippingParameters.setManifestId("347891314723499921");
            shipmentSearchKey.setRegularShippingParameters(regularShippingParameters);
            */

            //Get no-manifest shipments
            /*
            NoManifestShippingParameters noManifestShippingParameters = of.createNoManifestShippingParameters();
            noManifestShippingParameters.setNoManifest(true);
            noManifestShippingParameters.setDate(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-05-20"));
            noManifestShippingParameters.setLimit(150);
            shipmentSearchKey.setNoManifestShippingParameters(noManifestShippingParameters);
            */


            GetShipmentsRequest request = of.createGetShipmentsRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setShipmentSearchKey(shipmentSearchKey);

            //execute request
            GetShipmentsResponse response = port.getShipments(request);

            //retrieve values from response
            if(response.getMessages() == null){
                ShipmentsType shipments = response.getShipments();
                if(shipments.getShipmentId() != null && ! shipments.getShipmentId().isEmpty()){
                    for(String id : shipments.getShipmentId()){
                        System.out.println("Shipment Id: " + id);
                    }
                } else {
                    System.out.println("No shipments returned.");
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }

        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}
