package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.math.BigDecimal;
import java.util.Map;

/**
 * Sample code for the RequestShipmentRefund Canada Post service.
 * 
 * The RequestShipmentRefund service is used to request a refund for 
 * a shipment that has been transmitted. Making this call indicates 
 * that the previously printed label is spoiled or will otherwise not be used.
 * <p>
 * Note that these are requests for refund, as refunding a shipment requires 
 * proper verification before being actioned (i.e. ensure label has not been used, for example).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 *
 * For local testing only.
 */
public class RequestShipmentRefundClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());

            //build request
            ShipmentRefundRequestType shipmentRefundRequestType = new ShipmentRefundRequestType();
            shipmentRefundRequestType.setEmail("user@host.com");

            RequestShipmentRefundRequest request = new RequestShipmentRefundRequest();
            request.setMailedBy(mailedBy);
            request.setLocale("FR");
            request.setShipmentRefundRequest(shipmentRefundRequestType);

            //execute request
            RequestShipmentRefundResponse response = port.requestShipmentRefund(request);

            //retrieve values from response
            if(response.getMessages() == null){
                ShipmentRefundRequestInfoType shipmentRefundRequestInfoType = response.getShipmentRefundRequestInfo();

                System.out.println("Service Ticket ID: " + shipmentRefundRequestInfoType.getServiceTicketId());
                System.out.println("Service Ticket Date: " + shipmentRefundRequestInfoType.getServiceTicketDate());
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
